/**
 * ----------------------------------------------------- Utils 工具类 start -----------------------------------------------------
 * Add by JoyoDuan on 2017-08-16
 */
export declare class Utils {
    /**
     * 判断对象是否是undefined | null
     *
     * @param obj
     */
    static isNull(obj: Object | any): boolean;
    /**
     * 判断对象是否是undefined | null
     *
     * @param obj
     */
    static isNotNull(obj: Object | any): boolean;
    /**
     * 判断字符创是否为空
     * @param {string}  str 需要做判断的字符串
     * @return {boolean} return: true,为空；false,不为空
     */
    static isEmptyStr(str: string | number): boolean;
    /**
     * 判断字符创是否不为空
     * @param  {string}  str 需要做判断的字符串
     * @return {boolean}     return: true,不为空；false,为空
     * @author Add by JoyoDuan on 2017-08-16
     */
    static isNotEmptyStr(str: string | number): boolean;
    /**
     * 判断对象是否为空
     * @static
     * @param {Object} object 判断的对象
     * @return {boolean} 返回结果
     */
    static isEmptyObject(object: Object | any): boolean;
    /**
     * 判断对象是否不为空
     * @static
     * @param {Object} object 判断的对象
     * @return {boolean} 返回结果
     */
    static isNotEmptyObject(object: Object): boolean;
    /**
     * 判断json对象是否为空
     * @static
     * @param {*} json
     * @return {boolean}
     */
    static isEmptyJSON(json: any): boolean;
    /**
     * 判断json对象是否不为空
     * @static
     * @param {*} json
     * @return {boolean}
     */
    static isNotEmptyJSON(json: any): boolean;
    /**
     * 隐藏身份证号码、手机号码部分，以"*"代替
     *
     * typeof object === 'object': 取值：'number', 'boolean', 'string', 'function', 'object', 'undefined'
     *
     * @param str 字符串或number 如：'18083831850' || 18083831850
     * @param length Array 长度 ['左边', '中间', '右边'] '左边'为保留左边字符的长度，'中间'为显示symbol的个数，'右边'为保留右边的字符长度
     * @param symbol 隐藏符号，默认: '*'
     * @return 如：18083831850 => 180*****850
     */
    static evade(str: string | number, length?: Array<number>, symbol?: string): string;
    /**
     * 根据正则表达式获取字符串符合规则内容的索引
     *
     * 若不存在：return -1
     * 若存在：
     * 1: last = true, 返回str最后出现的位置
     * 2: last = false, 返回str第一次出现的位置
     *
     * @param str 要判断的字符串
     * @param regular 判断比较字符串的规则
     * @param last 是否返回最后一次出现的位置
     */
    static regularIndexOf(str: string, regular: RegExp, last?: boolean): number;
    /**
     * 获取字符串长度，汉字计数为2
     * @static
     * @param {string} str
     * @return {number}
     */
    static getStrLength(str: string): number;
    /**
     * 获取JSON对象的长度
     *
     * @param jsonObject
     */
    static getJSONLength(jsonObject: any): number;
    /**
     * 每调用一次，sequence加1
     * @param sequence
     */
    static getSequence: () => number;
    /**
     * 获取当前时间的时间戳
     * 获取当前时间的毫秒
     * @static
     * @return {number}
     */
    static getCurrentTime(): number;
    /**
     * 获取时间
     * 主要解决ios下new Date('1994-08-08 00:00:00')无效的问题
     *
     * @static
     * @return {number}
     */
    static getDate(date?: Date | string | number | any): Date;
    /**
     * Add by Joyo on 2017-08-16
     * 日期对象转为日期字符串
     * 使用格式：dateFormat(new Date(),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * yyyy-MM-dd HH:mm:ss 年：y，月：M，日：d，时：h，分：m，秒：s  ps:HH:24小时制，hh:12小时制
     * @param date 要格式的时间Date对象
     * @param pattern 输出格式,默认为:yyyy-MM-ddTHH:mm:ss+08:00，为了方便ion-date标签（因为ion-date标签只支持此格式）
     * @example dateFormat(new Date()) "2017-08-16"
     * @example dateFormat(new Date(),'yyyy-MM-dd') "2017-08-16"
     * @example dateFormat(new Date(),'yyyy-MM-dd HH:mm:ss') "2017-08-16 13:24:00" ps:HH:24小时制
     * @example dateFormat(new Date(),'yyyy-MM-dd hh:mm:ss') "2017-08-16 01:24:00" ps:hh:12小时制
     * @example dateFormat(new Date(),'hh:mm') "09:24"
     * @example dateFormat(new Date(),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * @example dateFormat(new Date('2017-08-16 13:24:00'),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * @return {string}
     */
    static dateFormat(date: Date | string | number | any, pattern?: string): string;
    /**
     * 获取日期差
     *
     * @param  startDate 开始日期
     * @param  endDate 结束日期
     * @param  compareType 比较类型('millisecond'/'second'/'minute'/'hour'/'day')，默认为'day'
     * @return  number 日期差
     */
    static dateCompare(startDate: Date | string | number | any, endDate: Date | string | number | any, compareType?: string): number;
    /**
     * 判断两个时间是否是同一天
     *
     * @Author JoyoDuan
     * @Date 2020/1/14
     * @Description:
     */
    static isSameDay(date: Date | string | number | any, basicDate?: Date | string | number | any): boolean;
    /**
     * 两个时间是否在同一周
     * 每周的第一天为星期一
     *
     * @Author JoyoDuan
     * @Date 2020/1/14
     * @Description:
     */
    static isSameWeek(date: Date | string | number | any, basicDate?: Date | string | number | any): boolean;
    /**
     * 传入时间，获取昨天、今天、明天、周、年月日
     *
     * @Author JoyoDuan
     * @Date 2020/1/15
     * @Description:
     */
    static getCalendarDay(date: Date | string | number | any, basicDate?: Date | string | number | any, pattern?: string, dayArray?: Array<string>, weekArray?: Array<string>): string;
    /**
     *  把url中的双斜杠替换为单斜杠
     *  如:http://localhost:8080//api//demo.替换后http://localhost:8080/api/demo
     * @param {string} url
     * @return {string}
     */
    static formatURL(url: string): string;
    /**
     * 转换post请求中URL str的"+"，转为"%2B"
     * /g为全局替换，global的缩写，\+为转义加号
     * @static
     * @return {string}
     */
    static formatPlus(str: string): string;
    /**
     * float精度处理
     *
     * @param floatNum float数字或字符串
     * @param dotDigit 保留小数点后几位 默认：0
     */
    static formatFloat(floatNum: number | string, dotDigit?: number): string;
    /**
     * 获取百分比
     *
     * @param num 当前数据
     * @param totalNum 总数据
     * @param isPercentSign 是否返回%字符，默认：true
     */
    static getPercent(num: number | string, totalNum: number | string, isPercentSign?: boolean): string | number;
    /**
     * 获取26个字母
     *
     * @static
     * @param {boolean} [isUpperCase=true] 是否是大写字母
     * @return {Array<string>}
     */
    static get26Letters(isUpperCase?: boolean): Array<string>;
    /**
     * 数组根据拼音首字母排序并分组
     *
     * 这里return any 而不是map，是因为JSON.parse(map)时有问题，而我们要使用的数据是JSON;
     *
     * @static
     * @param {Array<any>} data data中必须包含'name'属性
     * @return {Array<any>} 如果返回null，则表示当前浏览器不支持'localeCompare'方法，不能使用该方法
     */
    static letterSortGroup(data: Array<any>): any;
    /**
     * 将map转换成JSON对象
     *
     * @param map
     */
    static mapToJSONObject(map: Map<string, any>): any;
    /**
     * 将JSON对象或JSON字符串转换成map
     *
     * @param json
     */
    static jsonToMap(json: any | string): Map<string, any>;
    /**
     * json对象转为URL请求参数
     *
     * @Author JoyoDuan
     * @Date 2020/3/18
     * @Description:
     */
    static jsonToURI(json: any, symbol?: string, isEncodeURIComponent?: boolean): string;
    /**
     * 获取url的参数，并转为json
     *
     * @Author JoyoDuan
     * @Date 2020/3/18
     * @Description:
     */
    static uriToJSON(url: string, symbol?: string, isEncodeURIComponent?: boolean): any;
    /**
     * 将map转为数组
     *
     * map: Map<string, any>
     * return: array 二维数组 如: [[key, value], [key, value], ...]
     */
    static mapToArray(map: Map<string, any>): Array<Array<any>>;
    /**
     * 合并两个map
     * 合并后return targetMap
     *
     * 遍历sourceMap，把sourceMap的(key, value)赋给targetMap
     *
     * @param targetMap 目标对象(Map)
     * @param sourceMap 来源对象(Map)
     *
     * @return targetMap
     */
    static mergeMap(targetMap: Map<string, any>, sourceMap: Map<string, any>): Map<string, any>;
    /**
     * 将具有length属性的对象转换成数组
     *
     * @param obj 具有length、size属性，具有forEach方法，如：string map
     * @return arr 返回转换后的数组
     */
    static toArray(obj: Object | any): Array<any>;
    /**
     * 转换金额格式
     * 如：10000 --> 10,000.00
     */
    static toThousands(sum: number | string, dotNum?: number): string;
    /**
     * 生成32位的guid
     *
     * @return 32位guid字符串
     */
    static getGUID(length?: number): string;
    /**
     * 根据输入秒数，获取倒计时
     *
     * @param {number} seconds
     * @return {*}
     */
    static getCountdown(inputSeconds: number, showDay?: boolean): any;
    /**
     * 判断值是否是数字
     *
     * @param object
     * @return boolean
     */
    static isNumber(object: Object | any): boolean;
    /**
     * 判断是否为整数
     *
     * @param object 需要判断的数字
     */
    static isInteger(object: Object | any): boolean;
    /**
     * 输入内容是否是手机号码
     *
     * @param value 手机号码
     */
    static isPhone(value: string): boolean;
    /**
     * 输入内容是否是邮箱
     *
     * @param value 邮箱
     */
    static isEmail(value: string): boolean;
    /**
     * 输入内容是否是身份证格式
     *
     * @param value 身份证号码
     */
    static isIDCard(value: string): boolean;
    /**
     * 输入内容是否为中文
     *
     * @param value 字符串
     */
    static isChinese(value: string): boolean;
    /**
     * 输入内容是否为英文
     *
     * @param value 字符串
     */
    static isEnglish(value: string): boolean;
    /**
     * 输入内容是否包含特殊字符
     * 验证名称（不能含有特殊字符）, named
     *
     * @param value 字符串
     */
    static isNamed(value: string): boolean;
    /**
     * 验证多个以","隔开的数组名称
     * 验证名称（不能含有特殊字符）, named
     *
     * @param value 字符串
     */
    static isMultiNamed(value: string): boolean;
    /**
     * 将数字转成中文大写
     *
     * @param num
     * @return string
     */
    static digitUppercase(num: number | string): string;
    /**
     * 数组分组
     * 目前用于表情包分组
     * 使用了Math.ceil() 四舍五入，取大值
     * example: 15.1 --> 16  15.6 --> 16  15 --> 15
     *
     * @param array 要分组的数组
     * @param groupNum 分组后的每个数组数量
     */
    static arrayGroup(array: Array<any>, eachGroupNum?: number): Array<any>;
    /**
     * 获取随机字符串
     *
     * @static
     * @param {number} [length=32]
     * @param {Array<string>} [randomArray]
     * @return {string}
     */
    static getRandomStr(length?: number, randomArray?: Array<string>): string;
    /**
     * 获取随机字符串
     * return Math.random().toString(36).substring(2);
     *
     * @Author JoyoDuan
     * @Date 2020/5/9
     * @Description:
     */
    static getNonceStr(radix?: number): string;
    /**
     * 内容是否包含敏感词汇
     *
     * @static
     * @param {string} str 检测的内容
     * @param {Array<string>} forbiddenArray 被禁止的词汇
     * @return {boolean} 是否包含敏感词汇 true: 包含该  false: 不包含
     */
    static isSensitive(str: string, forbiddenArray: Array<string>): boolean;
    /**
     * 获取含有搜索条件的数组
     *
     * @static
     * @param {Array<any>} array 原数组
     * @param {string} delimiter 搜索条件的分隔符
     * @return {Array<any>} 转换后的数组
     */
    static getSearchArray(array: Array<any>, delimiter?: string, field?: string): Array<any>;
    /**
     * 获取含有搜索条件的item
     *
     * @static
     * @param {any} item 原item
     * @param {string} delimiter 搜索条件的分隔符
     * @return {any} 转换后的item
     */
    static getSearchItem(item: any, delimiter?: string, field?: string): any;
    /**
     * 根据日期获取星期几
     *
     * @static
     * @param {string} date 日期
     * @param weekDays 自定义星期数组
     * @return {string} 当前时间是星期几
     */
    static getWeekDay(date?: Date | string | number | any, weekDays?: Array<string>): string;
    /**
     * 获取身份证信息
     *
     * @param IDCard 身份证号
     * @param type 返回类型: all | birthday | age | sex， 默认为: all
     * @return {any} result: {birthday, age, sex} | birthday | age | sex
     */
    static getIDCardInfo(IDCard: string, type?: string): any;
    /**
     * 扩展参数，返回userOptions
     * 注意，该方法会改变userOptions的源对象，如果不希望改变，请传入{...userOptions}或者isRelated: false
     * isRelated为false时，内部方法会调用Utils.deepClone方法，所以性能稍差一些
     *
     * @Author JoyoDuan
     * @Date 2020/5/14
     * @Description:
     * @param defaultOptions 默认配置
     * @param userOptions 用户配置
     * @param isRelated 默认为true，改变内容是否关联userOptions，如果为true，则传入的userOptions会改变
     * @returns isRelated为true返回userOptions对象，否则返回全新对象
     */
    static extend(defaultOptions: Object | any, userOptions: Object | any, isRelated?: boolean): any;
    /**
     * 扩展基础（根基）参数，返回basicOptions
     * 注意，该方法会改变basicOptions的源对象，如果不希望改变，请传入{...basicOptions}或者isRelated: false
     * isRelated为false时，内部方法会调用Utils.deepClone方法，所以性能稍差一些
     *
     * @Author JoyoDuan
     * @Date 2020/5/14
     * @Description:
     * @param basicOptions 默认基础配置
     * @param userOptions 用户配置
     * @param isRelated 默认为true，改变内容是否关联userOptions，如果为true，则传入的userOptions会改变
     * @returns isRelated为true返回userOptions对象，否则返回全新对象
     */
    static extendBasic(basicOptions: Object | any, userOptions: Object | any, isRelated?: boolean): any;
    /**
     * 深度复制对象
     * 该方法不能拷贝 时间：Date，正则: RegExp，如有这些属性，请使用 deepClone()方法
     *
     * @param obj
     */
    static deepCopy(obj: Object | any): any;
    /**
     * 深度拷贝对象
     * 该方法支持拷贝 时间：Date，正则: RegExp
     *
     * @static
     * @param {*} obj
     * @return {*}
     */
    static deepClone(obj: any): any;
    /**
     * 在URL/URI后边添加后缀
     * 后缀格式：?Math.random() 如: https://www.baidu.com/JoyoDuan.jpg?0.88001
     *
     * @param url
     */
    static tailURL(url: string): string;
    /**
     * 比较版本号
     * Utils.versionCompare('1.0.358_20180820090554', '1.0.358_20180820090553') = 1
     *
     * @static
     * @param {string} version1
     * @param {string} version2
     * @return {number} 0代表相等，1代表version1大，-1代表version2
     */
    static versionCompare(version1: string, version2: string): number;
    /**
     * 是否是手机环境
     *
     * @Author JoyoDuan
     * @Date 2020/4/14
     * @Description:
     */
    static isMobile(): boolean;
    /**
     * 是否是ios设备
     *
     * @return true: 是  false: 否
     */
    static isIOS(): boolean;
    /**
     * 是否是Safari设备
     *
     * @return true: 是  false: 否
     */
    static isSafari(): boolean;
    /**
     * 是否是Android设备
     *
     * @return true: 是  false: 否
     */
    static isAndroid(): boolean;
    /**
     * 是否是PC环境
     *
     * @return true: 是  false: 否
     */
    static isPC(): boolean;
    /**
     * 获取子元素
     *
     * @param element 元素
     * @param cssClass classList.contains(cssClass) 元素样式
     * @param selectors element.querySelector(selectors) 查找元素的样式
     *
     * @Author JoyoDuan
     * @Date 2019/12/16
     * @Description:
     */
    static getChildElement(element: HTMLElement, cssClass?: string, selectors?: string): HTMLElement | any;
    /**
     * 获取元素样式属性的值
     *
     * @param element 元素
     * @param styleName 属性 如：marginTop、height
     * @param pseudoElt getComputedStyle()中的第二个参数 指定一个要匹配的伪元素的字符串。必须对普通元素省略（或null）
     */
    static getElementStyleValue(element: Element | any, styleName: string | any, pseudoElt?: string | null): number;
    /**
     * 将字母转成小写
     *
     * @param str 需要转换的字符串
     */
    static toLowerCase(str: string): string;
    /**
     * 将字母转成大写
     *
     * @param str 需要转换的字符串
     */
    static toUpperCase(str: string): string;
    /**
     * 将字母转成小写
     * 少数语言(如土耳其语言)会为Unicode大小写转换应用特殊的规则
     *
     * @param str 需要转换的字符串
     */
    static toLocaleLowerCase(str: string): string;
    /**
     * 将字母转成大写
     * 少数语言(如土耳其语言)会为Unicode大小写转换应用特殊的规则
     *
     * @param str 需要转换的字符串
     */
    static toLocaleUpperCase(str: string): string;
    /**
     * 根据文件路径filePath获取文件名
     *
     * @param string path
     * @returns string
     */
    static getFileName(path: string): string;
    /**
     * 根据文件路径获取文件后缀
     *
     * @param string path
     * @returns string
     */
    static getFileSuffix(path: string): string;
    /**
     * 计时器
     * 执行回调后会clearTimeout
     *
     * @param handler 执行函数
     * @param time 计时器时间
     * @Author JoyoDuan
     * @Date 2019/12/17
     * @Description:
     */
    static setTimeout(handler: () => void, time?: number): void;
    /**
     * 自定义实现eval()方法
     * eval方法只能在非严格模式中进行使用，在use strict中是不允许使用这个方法的。
     *
     * @Author JoyoDuan
     * @Date 2020/6/11
     * @Description:
     */
    static eval(x: string | any): any;
    /**
     * 删除对象中的空属性（或键值对）
     *
     * @Author JoyoDuan
     * @Date 2020/7/7
     * @Description:
     */
    static deleteEmptyProperty(obj: Object | Array<any> | any): any;
    /**
     * 中文转Unicode
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    static encodeUnicode(str: string): string;
    /**
     * Unicode转中文
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    static decodeUnicode(str: string): string;
    /**
     * Ascii 转 Unicode
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    static A2U(str: string): string;
    /**
     * Unicode 转 Ascii
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    static U2A(str: string): string;
}
