/**
 * ----------------------------------------------------- Utils 工具类 start -----------------------------------------------------
 * Add by JoyoDuan on 2017-08-16
 */
var Utils = (function () {
    function Utils() {
    }
    /**
     * 判断对象是否是undefined | null
     *
     * @param obj
     */
    Utils.isNull = function (obj) {
        return obj === undefined || obj === null;
    };
    /**
     * 判断对象是否是undefined | null
     *
     * @param obj
     */
    Utils.isNotNull = function (obj) {
        return !this.isNull(obj);
    };
    /**
     * 判断字符创是否为空
     * @param {string}  str 需要做判断的字符串
     * @return {boolean} return: true,为空；false,不为空
     */
    Utils.isEmptyStr = function (str) {
        if (this.isNull(str))
            return true;
        // 防止any类型报错
        str = str.toString();
        if (str.trim() === '' || str.trim().length === 0)
            return true;
        return false;
    };
    /**
     * 判断字符创是否不为空
     * @param  {string}  str 需要做判断的字符串
     * @return {boolean}     return: true,不为空；false,为空
     * @author Add by JoyoDuan on 2017-08-16
     */
    Utils.isNotEmptyStr = function (str) {
        return !this.isEmptyStr(str);
    };
    /**
     * 判断对象是否为空
     * @static
     * @param {Object} object 判断的对象
     * @return {boolean} 返回结果
     */
    Utils.isEmptyObject = function (object) {
        // 空对象
        if (this.isNull(object)) {
            return true;
        }
        for (var key in object) {
            key = key; // 因为es6要校验格式，因此使用一次key
            return false; // key存在，不是空对象，返回false
        }
        // Map对象，for in时key为undefined，Object.keys()为[]，所以单独处理
        if (object.size !== undefined) {
            return object.size === 0;
        }
        // 判断空json对象、数字时
        if (this.isEmptyStr(object.toString()) || JSON.stringify(object) === '{}') {
            return true;
        }
        return false;
    };
    /**
     * 判断对象是否不为空
     * @static
     * @param {Object} object 判断的对象
     * @return {boolean} 返回结果
     */
    Utils.isNotEmptyObject = function (object) {
        return !this.isEmptyObject(object);
    };
    /**
     * 判断json对象是否为空
     * @static
     * @param {*} json
     * @return {boolean}
     */
    Utils.isEmptyJSON = function (json) {
        // if (JSON.stringify(json) === undefined || JSON.stringify(json) === 'null' || JSON.stringify(json) === '{}' || JSON.stringify(json) === '""') {
        if (this.isNull(json) || JSON.stringify(json) === '{}') {
            return true; // 为空，返回true
        }
        return false; // 不为空，返回false
    };
    /**
     * 判断json对象是否不为空
     * @static
     * @param {*} json
     * @return {boolean}
     */
    Utils.isNotEmptyJSON = function (json) {
        return !this.isEmptyJSON(json);
    };
    /**
     * 隐藏身份证号码、手机号码部分，以"*"代替
     *
     * typeof object === 'object': 取值：'number', 'boolean', 'string', 'function', 'object', 'undefined'
     *
     * @param str 字符串或number 如：'18083831850' || 18083831850
     * @param length Array 长度 ['左边', '中间', '右边'] '左边'为保留左边字符的长度，'中间'为显示symbol的个数，'右边'为保留右边的字符长度
     * @param symbol 隐藏符号，默认: '*'
     * @return 如：18083831850 => 180*****850
     */
    Utils.evade = function (str, length, symbol) {
        if (symbol === void 0) { symbol = '*'; }
        if (Utils.isEmptyStr(str) || str === 0) {
            return '';
        }
        // 不传入则取默认值
        if (!length || length.length <= 0) {
            length = [3, 4, 4];
        }
        // 去空格
        str = str.toString().trim();
        // 字符串长度
        var strLength = str.length;
        var result = '', leftLength = length[0], centerLength = length[1], rightLength = length[2];
        // 字符串不够截，等比减小左右两边的长度
        while (strLength <= (leftLength + rightLength)) {
            leftLength = Math.floor(leftLength / 2);
            rightLength = Math.floor(rightLength / 2);
        }
        // 处理左右截取长度后，可能存在两边都为0了
        // 字符长度大于1才截，否则左右两边都不保留，显示'*'
        if (leftLength === 0 && rightLength === 0 && strLength > 1) {
            leftLength = 1;
        }
        // 显示symbol不存在或为0，则表示取默认剩下长度
        if (centerLength === 0 || !centerLength) {
            centerLength = strLength - (leftLength + rightLength);
        }
        // 左边
        result = str.substr(0, leftLength);
        // 中间以*号代替
        for (var i = 0; i < centerLength; i++) {
            result += symbol;
        }
        // 右边
        result += str.substr(strLength - rightLength, strLength);
        return result;
    };
    /**
     * 根据正则表达式获取字符串符合规则内容的索引
     *
     * 若不存在：return -1
     * 若存在：
     * 1: last = true, 返回str最后出现的位置
     * 2: last = false, 返回str第一次出现的位置
     *
     * @param str 要判断的字符串
     * @param regular 判断比较字符串的规则
     * @param last 是否返回最后一次出现的位置
     */
    Utils.regularIndexOf = function (str, regular, last) {
        if (last === void 0) { last = false; }
        // 内容符合正则表达式
        if (this.isNotEmptyStr(str) && str.match(regular)) {
            var chars = ['~', '!', '@', '#', '$', '%', '^', '&', '*'];
            // str.match(regular)[0].length传入，保证返回的索引正确
            var replaceStr = this.getRandomStr(str.match(regular)[0].length, chars);
            // 保证生成的临时替换str不能再在判断的字符串中出现
            while (str.indexOf(replaceStr) !== -1) {
                replaceStr = this.getRandomStr(str.match(regular)[0].length, chars);
            }
            // 将符合的内容替换成特殊符号
            var regularStr = str.replace(regular, replaceStr);
            if (!last)
                return regularStr.indexOf(replaceStr); // 返回第一次出现的index
            return regularStr.lastIndexOf(replaceStr); // 返回最后出现的index
        }
        else {
            return -1;
        }
    };
    /**
     * 获取字符串长度，汉字计数为2
     * @static
     * @param {string} str
     * @return {number}
     */
    Utils.getStrLength = function (str) {
        if (this.isEmptyStr(str))
            return 0;
        var length = 0;
        for (var i = 0; i < str.length; i++) {
            str.charCodeAt(i) > 255 ? length += 2 : length++;
        }
        return length;
    };
    /**
     * 获取JSON对象的长度
     *
     * @param jsonObject
     */
    Utils.getJSONLength = function (jsonObject) {
        // 空JSON， return 0
        if (this.isEmptyJSON(jsonObject))
            return 0;
        var length = 0;
        for (var json in jsonObject) {
            json = json; // 因为es6要校验格式，因此使用一次json
            length++;
        }
        return length;
    };
    // 不能这样写，这样永远返回的都是2
    /* static getSequence(): number
    {
      let sequence = 1;
      return ++sequence;
    } */
    /**
     * 获取当前时间的时间戳
     * 获取当前时间的毫秒
     * @static
     * @return {number}
     */
    Utils.getCurrentTime = function () {
        return new Date().getTime();
    };
    /**
     * 获取时间
     * 主要解决ios下new Date('1994-08-08 00:00:00')无效的问题
     *
     * @static
     * @return {number}
     */
    Utils.getDate = function (date) {
        // date允许string | number类型，所以date需加入any类型，否则date.getDay()会编译错误
        // 没有传入时间
        if (this.isEmptyObject(date)) {
            return new Date();
        }
        // 日期格式，直接返回
        if (date instanceof Date) {
            return date;
        }
        // 传入字符串或时间戳
        if (typeof date === 'string' || typeof date === 'number' || typeof date !== 'object') {
            // 处理safari、ios上 new Date('2019-04-12 17:50:00')会报时间NAN safari上不支持'-'
            if (typeof date === 'string') {
                // 解决ios 8601时间转换为东八区（中国）标准时间new Date('2019-04-14T16:00:00Z')会加8个小时
                // new Date('2019-04-14T16:00:00Z') --> Mon Apr 15 2019 00:00:00 GMT+0800 (中国标准时间) 加了8个小时
                if (date.indexOf('T') !== -1 && date.indexOf('Z') !== -1) {
                    // 删掉T 和 Z，因为下边有new Date()
                    date = date.replace(/T/ig, ' ').replace(/Z/ig, '');
                }
                else if (date.indexOf('T') !== -1 && date.indexOf('+08:00') !== -1) {
                    // 删掉T 和 +08:00，因为下边有转换ios。 new Date('2019/10/25 14:31:51')会输出Invalid Date
                    date = date.replace(/T/ig, ' ').replace(/\+08:00/ig, '');
                }
                // ios设备，将-转换为/，否则new Date(date)不正确  如：1994-08-08 00:00:00 --> 1994/08/08 00:00:00
                if (this.isIOS() || this.isSafari()) {
                    date = date.replace(/-/g, '/');
                }
            }
            return new Date(date);
        }
        return new Date(date);
    };
    /**
     * Add by Joyo on 2017-08-16
     * 日期对象转为日期字符串
     * 使用格式：dateFormat(new Date(),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * yyyy-MM-dd HH:mm:ss 年：y，月：M，日：d，时：h，分：m，秒：s  ps:HH:24小时制，hh:12小时制
     * @param date 要格式的时间Date对象
     * @param pattern 输出格式,默认为:yyyy-MM-ddTHH:mm:ss+08:00，为了方便ion-date标签（因为ion-date标签只支持此格式）
     * @example dateFormat(new Date()) "2017-08-16"
     * @example dateFormat(new Date(),'yyyy-MM-dd') "2017-08-16"
     * @example dateFormat(new Date(),'yyyy-MM-dd HH:mm:ss') "2017-08-16 13:24:00" ps:HH:24小时制
     * @example dateFormat(new Date(),'yyyy-MM-dd hh:mm:ss') "2017-08-16 01:24:00" ps:hh:12小时制
     * @example dateFormat(new Date(),'hh:mm') "09:24"
     * @example dateFormat(new Date(),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * @example dateFormat(new Date('2017-08-16 13:24:00'),'yyyy-MM-ddTHH:mm:ss+08:00') "2017-08-16T13:24:00+08:00"
     * @return {string}
     */
    Utils.dateFormat = function (date, pattern) {
        if (pattern === void 0) { pattern = 'yyyy-MM-ddTHH:mm:ss+08:00'; }
        // date允许string | number类型，所以date需加入any类型，否则date.getDay()会编译错误
        // 没有传入时间
        if (this.isEmptyObject(date)) {
            return '';
        }
        // 传入字符串或时间戳，转换符合ios的时间
        date = this.getDate(date);
        // 定义时间
        var time = {
            Year: 0,
            TYear: '0',
            Month: 0,
            TMonth: '0',
            Day: 0,
            TDay: '0',
            Hour: 0,
            THour: '0',
            hour: 0,
            Thour: '0',
            Minute: 0,
            TMinute: '0',
            Second: 0,
            TSecond: '0',
            Millisecond: 0
        };
        time.Year = date.getFullYear(); // 完整的年份
        time.TYear = String(time.Year).substr(2); // 两位数的年份
        time.Month = date.getMonth() + 1; // 如1月，而不是01月。"+1"是因为月份是从"0"开始的
        time.TMonth = time.Month < 10 ? '0' + time.Month : String(time.Month); // 如01月，而不是1月
        time.Day = date.getDate(); // 如1日，而不是01日
        time.TDay = time.Day < 10 ? '0' + time.Day : String(time.Day); // 如01日，而不是1日
        time.Hour = date.getHours(); // 如24时，而不是12时
        time.THour = time.Hour < 10 ? '0' + time.Hour : String(time.Hour); // 如01时，而不是1时
        time.hour = time.Hour < 13 ? time.Hour : time.Hour - 12; // 如01时，而不是13时
        time.Thour = time.hour < 10 ? '0' + time.hour : String(time.hour); // 如01时，而不是1时
        time.Minute = date.getMinutes(); // 如1分，而不是01分
        time.TMinute = time.Minute < 10 ? '0' + time.Minute : String(time.Minute); // 如01分，而不是1分
        time.Second = date.getSeconds(); // 如1秒，而不是01秒
        time.TSecond = time.Second < 10 ? '0' + time.Second : String(time.Second); // 如01秒，而不是1秒
        time.Millisecond = date.getMilliseconds(); // 毫秒
        // 替换时间格式中的时间
        var result = pattern.replace(/yyyy/ig, String(time.Year)) // 如2017
            .replace(/yyy/ig, String(time.Year)) // 如2017
            .replace(/yy/ig, String(time.TYear)) // 如17
            .replace(/y/ig, String(time.TYear)) // 如17
            .replace(/MM/g, String(time.TMonth)) // 如01月
            .replace(/M/g, String(time.Month)) // 如1月
            .replace(/dd/ig, String(time.TDay)) // 如01日
            .replace(/d/ig, String(time.Day)) // 如1日
            .replace(/HH/g, String(time.THour)) // 如01时，24小时制
            .replace(/H/g, String(time.Hour)) // 如01时，24小时制
            .replace(/hh/g, String(time.Thour)) // 01时，12小时制
            .replace(/h/g, String(time.hour)) // 1时，12小时制
            .replace(/mm/g, String(time.TMinute))
            .replace(/m/g, String(time.Minute))
            .replace(/ss/ig, String(time.TSecond))
            .replace(/s/ig, String(time.Second))
            .replace(/fff/ig, String(time.Millisecond));
        return result;
    };
    /**
     * 获取日期差
     *
     * @param  startDate 开始日期
     * @param  endDate 结束日期
     * @param  compareType 比较类型('millisecond'/'second'/'minute'/'hour'/'day')，默认为'day'
     * @return  number 日期差
     */
    Utils.dateCompare = function (startDate, endDate, compareType) {
        if (compareType === void 0) { compareType = 'day'; }
        // startDate和endDate允许string | number类型，所以date需加入any类型，否则date.getDay()会编译错误
        // 没有传入时间
        if (this.isEmptyObject(startDate) || this.isEmptyObject(endDate)) {
            return 0;
        }
        // 传入字符串
        startDate = this.getDate(startDate);
        endDate = this.getDate(endDate);
        var num = 1;
        switch (compareType) {
            case 'millisecond':
                num = 1;
                break;
            case 'second':
                num = 1000;
                break;
            case 'minute':
                num = 1000 * 60;
                break;
            case 'hour':
                num = 1000 * 3600;
                break;
            case 'day':
                num = 1000 * 3600 * 24;
                break;
            default:
                break;
        }
        return (endDate.getTime() - startDate.getTime()) / num;
    };
    /**
     * 判断两个时间是否是同一天
     *
     * @Author JoyoDuan
     * @Date 2020/1/14
     * @Description:
     */
    Utils.isSameDay = function (date, basicDate) {
        // 没有传入时间
        if (this.isEmptyObject(date)) {
            return false;
        }
        // 转换时间
        date = this.getDate(date);
        basicDate = this.getDate(basicDate);
        return (date.setHours(0, 0, 0, 0) === basicDate.setHours(0, 0, 0, 0));
    };
    /**
     * 两个时间是否在同一周
     * 每周的第一天为星期一
     *
     * @Author JoyoDuan
     * @Date 2020/1/14
     * @Description:
     */
    Utils.isSameWeek = function (date, basicDate) {
        // 没有传入时间
        if (this.isEmptyObject(date)) {
            return false;
        }
        // 转换时间
        date = this.getDate(date).getTime();
        basicDate = this.getDate(basicDate).getTime();
        // 24小时的毫秒数
        var oneDayTime = 24 * 60 * 60 * 1000;
        var dateCount = parseInt((date / oneDayTime).toString(), 10);
        var basicDateCount = parseInt((basicDate / oneDayTime).toString(), 10);
        return parseInt(((dateCount + 4) / 7).toString(), 10) === parseInt(((basicDateCount + 4) / 7).toString(), 10);
    };
    /**
     * 传入时间，获取昨天、今天、明天、周、年月日
     *
     * @Author JoyoDuan
     * @Date 2020/1/15
     * @Description:
     */
    Utils.getCalendarDay = function (date, basicDate, pattern, dayArray, weekArray) {
        // 没有传入时间
        if (this.isEmptyObject(date)) {
            return '';
        }
        // 转换时间
        date = this.getDate(date);
        // 今天（根基）时间的截止时间
        basicDate = new Date(this.getDate(basicDate).setHours(0, 0, 0, 0));
        // 日期格式
        if (this.isEmptyStr(pattern)) {
            pattern = 'yyyy年MM月dd日';
        }
        // 不是同一年，直接返回年月日
        if (date.getFullYear() !== basicDate.getFullYear()) {
            return this.dateFormat(date, pattern);
        }
        // 同年中不需要显示年
        if (pattern.indexOf('y') !== -1) {
            pattern = pattern.substring(pattern.lastIndexOf('y') + 2, pattern.length);
        }
        // 没有传入日期或者传入日期数组不是奇数
        if (this.isEmptyObject(dayArray) || dayArray.length % 2 !== 0) {
            dayArray = ['昨天', '今天', '明天'];
        }
        // 周
        if (this.isEmptyObject(weekArray) || weekArray.length !== 7) {
            weekArray = ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六'];
        }
        // 设置时间到00:00:00:000
        var resetDate = date.setHours(0, 0, 0, 0);
        // 日期差（向上取整），天
        var dayCompare = this.dateCompare(basicDate, resetDate, 'day');
        // 日期差绝对值
        var dayCompareAbs = Math.abs(dayCompare);
        // 数组中间值的左、右长度，数组从0开始，所以向下取整
        var halfLength = Math.floor(dayArray.length / 2);
        // 超出dayArray
        if (dayCompareAbs > halfLength) {
            // 在同一周
            if (this.isSameWeek(basicDate, date)) {
                // 在同一周
                return this.getWeekDay(date, weekArray);
            }
            // 不在同一周， 返回日期
            return this.dateFormat(date, pattern);
        }
        // 今天
        if (dayCompareAbs === 0) {
            var toDay = dayArray[halfLength];
            if (this.isEmptyStr(toDay)) {
                return this.dateFormat(date, pattern);
            }
            return toDay;
        }
        // 未来
        if (dayCompare >= 1) {
            var future = dayArray[dayCompareAbs + halfLength];
            if (this.isEmptyStr(future)) {
                return this.dateFormat(date, pattern);
            }
            return future;
        }
        // 以前
        var before = dayArray[halfLength - dayCompareAbs];
        if (this.isEmptyStr(before)) {
            return this.dateFormat(date, pattern);
        }
        return before;
    };
    /**
     *  把url中的双斜杠替换为单斜杠
     *  如:http://localhost:8080//api//demo.替换后http://localhost:8080/api/demo
     * @param {string} url
     * @return {string}
     */
    Utils.formatURL = function (url) {
        if (this.isEmptyStr(url))
            return '';
        var index = 0;
        if (url.startsWith('https')) {
            index = 8;
        }
        else if (url.startsWith('http')) {
            index = 7;
        }
        // /g表示全局, "\"是转义字符
        return url.substring(0, index) + url.substring(index).replace(/\/\//g, '/');
    };
    /**
     * 转换post请求中URL str的"+"，转为"%2B"
     * /g为全局替换，global的缩写，\+为转义加号
     * @static
     * @return {string}
     */
    Utils.formatPlus = function (str) {
        if (this.isEmptyStr(str))
            return '';
        return str.replace(/\+/g, '%2B');
    };
    /**
     * float精度处理
     *
     * @param floatNum float数字或字符串
     * @param dotDigit 保留小数点后几位 默认：0
     */
    Utils.formatFloat = function (floatNum, dotDigit) {
        if (dotDigit === void 0) { dotDigit = 0; }
        if (this.isEmptyStr(floatNum)) {
            return '0';
        }
        floatNum = floatNum.toString();
        var dotSplit = floatNum.indexOf('.');
        var dotLength = floatNum.substr(dotSplit + 1).length;
        var pow = Math.pow(10, dotLength);
        return (parseInt((parseFloat(floatNum) * pow).toString(), 10) / pow).toFixed(dotDigit);
    };
    /**
     * 获取百分比
     *
     * @param num 当前数据
     * @param totalNum 总数据
     * @param isPercentSign 是否返回%字符，默认：true
     */
    Utils.getPercent = function (num, totalNum, isPercentSign) {
        if (isPercentSign === void 0) { isPercentSign = true; }
        if (this.isEmptyStr(num) || this.isEmptyStr(totalNum)) {
            return '-';
        }
        num = parseFloat(num.toString());
        totalNum = parseFloat(totalNum.toString());
        if (!this.isNumber(num) || !this.isNumber(totalNum)) {
            return '-';
        }
        // 是否显示 '%'
        if (isPercentSign) {
            return totalNum <= 0 ? '0%' : (Math.round(num / totalNum * 10000) / 100.00 + '%');
        }
        else {
            return totalNum <= 0 ? 0 : (Math.round(num / totalNum * 10000) / 100.00);
        }
    };
    /**
     * 获取26个字母
     *
     * @static
     * @param {boolean} [isUpperCase=true] 是否是大写字母
     * @return {Array<string>}
     */
    Utils.get26Letters = function (isUpperCase) {
        // 多种方法参考
        // 26个大写字母
        // let chars: string = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        // for (let i = 0; i < chars.length; i ++)
        // {
        //   this.indexes.push(chars.charAt(i));
        // }
        // 该方法等同于上面的方法
        // for (let i = 0; i < 26; i ++)
        // {
        //   this.indexes.push(String.fromCharCode(65 + i));
        // }
        // this.indexes = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');
        if (isUpperCase === void 0) { isUpperCase = true; }
        // 字母数组
        var letters = [];
        // 是否是大写
        if (!isUpperCase) {
            for (var i = 0; i < 26; i++) {
                letters.push(String.fromCharCode(65 + i).toLowerCase());
            }
        }
        else {
            for (var i = 0; i < 26; i++) {
                letters.push(String.fromCharCode(65 + i));
            }
        }
        return letters;
    };
    /**
     * 数组根据拼音首字母排序并分组
     *
     * 这里return any 而不是map，是因为JSON.parse(map)时有问题，而我们要使用的数据是JSON;
     *
     * @static
     * @param {Array<any>} data data中必须包含'name'属性
     * @return {Array<any>} 如果返回null，则表示当前浏览器不支持'localeCompare'方法，不能使用该方法
     */
    Utils.letterSortGroup = function (data) {
        if (this.isEmptyObject(data) || !String.prototype.localeCompare) {
            return {};
        }
        // 23个拼音字母数组
        var letters = 'ABCDEFGHJKLMNOPQRSTWXYZ'.split('');
        // 23个拼音字母对应的中文数组
        var zh = '阿八嚓哒妸发旮哈讥咔垃痳拏噢妑七呥扨它穵夕丫帀'.split('');
        // 用于接收结果的Map数组
        // let result: Map<string, Array<any>> = new Map<string, Array<any>>();
        var result = {};
        // 根据字母分组的内容
        var section = {
            letter: '',
            data: []
        };
        // 循环字母数组，用于根据字母分组
        letters.forEach(function (letter, letterIndex) {
            // 每个字母对应的数据分组
            section = { letter: letter, data: [] };
            // 循环数据数组
            data.forEach(function (value) {
                // a.localeCompare(b) = 1是a在b的后面； a.localeCompare(b) = 0是a和b并排； a.localeCompare(b) = -1是a在b的前面
                if (value.name.localeCompare(zh[letterIndex], 'zh') >= 0 && value.name.localeCompare(zh[letterIndex + 1], 'zh') === -1) {
                    section.data.push(value);
                }
            });
            // 如果存在数据则进行排序
            if (section.data.length > 0) {
                // 对分组后的数组进行排序
                section.data.sort(function (a, b) {
                    return a.name.localeCompare(b.name, 'zh');
                });
            }
            // result.set(section.letter, section.data);
            result[section.letter] = section.data;
        });
        return result;
    };
    /**
     * 将map转换成JSON对象
     *
     * @param map
     */
    Utils.mapToJSONObject = function (map) {
        var result = {};
        if (this.isEmptyObject(map)) {
            return result;
        }
        map.forEach(function (value, key) {
            result[key] = value;
        });
        return result;
    };
    /**
     * 将JSON对象或JSON字符串转换成map
     *
     * @param json
     */
    Utils.jsonToMap = function (json) {
        var map = new Map();
        if (this.isEmptyJSON(json)) {
            return map;
        }
        // 传入的是js字符串
        if (typeof json === 'string') {
            json = JSON.parse(json);
        }
        for (var key in json) {
            map.set(key, json[key]);
        }
        return map;
    };
    /**
     * json对象转为URL请求参数
     *
     * @Author JoyoDuan
     * @Date 2020/3/18
     * @Description:
     */
    Utils.jsonToURI = function (json, symbol, isEncodeURIComponent) {
        if (isEncodeURIComponent === void 0) { isEncodeURIComponent = true; }
        try {
            var tempArr = [];
            if (this.isEmptyStr(symbol)) {
                symbol = '&';
            }
            if (isEncodeURIComponent) {
                var key = '', value = null;
                for (var k in json) {
                    key = encodeURIComponent(k);
                    value = encodeURIComponent(json[k]);
                    tempArr.push(key + '=' + value);
                }
            }
            else {
                for (var k in json) {
                    tempArr.push(k + '=' + json[k]);
                }
            }
            return tempArr.join(symbol ? symbol : '&');
        }
        catch (e) {
            return '';
        }
    };
    /**
     * 获取url的参数，并转为json
     *
     * @Author JoyoDuan
     * @Date 2020/3/18
     * @Description:
     */
    Utils.uriToJSON = function (url, symbol, isEncodeURIComponent) {
        if (isEncodeURIComponent === void 0) { isEncodeURIComponent = true; }
        try {
            var index = url.indexOf('?');
            // 完整的路径
            if (index > -1) {
                url = url.match(/\?([^#]+)/)[1];
            }
            var json = {}, arr = url.split('&'), subArr = [], key = '', value = '';
            if (this.isEmptyStr(symbol)) {
                symbol = '&';
            }
            if (isEncodeURIComponent) {
                for (var i = 0; i < arr.length; i++) {
                    subArr = arr[i].split('=');
                    key = decodeURIComponent(subArr[0]);
                    value = decodeURIComponent(subArr[1]);
                    json[key] = value;
                }
            }
            else {
                for (var i = 0; i < arr.length; i++) {
                    subArr = arr[i].split('=');
                    key = subArr[0];
                    value = subArr[1];
                    json[key] = value;
                }
            }
            return json;
        }
        catch (e) {
            return null;
        }
    };
    /**
     * 将map转为数组
     *
     * map: Map<string, any>
     * return: array 二维数组 如: [[key, value], [key, value], ...]
     */
    Utils.mapToArray = function (map) {
        var array = [];
        for (var key in map) {
            array.push([key, map.get(key)]);
        }
        return array;
    };
    /**
     * 合并两个map
     * 合并后return targetMap
     *
     * 遍历sourceMap，把sourceMap的(key, value)赋给targetMap
     *
     * @param targetMap 目标对象(Map)
     * @param sourceMap 来源对象(Map)
     *
     * @return targetMap
     */
    Utils.mergeMap = function (targetMap, sourceMap) {
        if (this.isEmptyObject(targetMap) && this.isEmptyObject(sourceMap)) {
            return new Map();
        }
        if (this.isEmptyObject(targetMap))
            return sourceMap;
        if (this.isEmptyObject(sourceMap))
            return targetMap;
        sourceMap.forEach(function (value, key) {
            targetMap.set(key, value);
        });
        return targetMap;
    };
    /**
     * 将具有length属性的对象转换成数组
     *
     * @param obj 具有length、size属性，具有forEach方法，如：string map
     * @return arr 返回转换后的数组
     */
    Utils.toArray = function (obj) {
        if (this.isEmptyObject(obj)) {
            return new Array();
        }
        try {
            return [].slice.call(obj); // 等同于下
            // return Array.prototype.slice.call(obj);
        }
        catch (error) {
            var arr_1 = [];
            var index_1 = 0;
            [].forEach.call(obj, function (value, i) {
                // arr.push(value);
                arr_1[index_1] = value; // 据说这样比push快
                index_1++;
            });
            // 等同于上
            // for (let i = 0; i < obj.length; i ++) {
            //   // arr.push(obj[i]);
            //   arr[i] = obj[i]; // 据说这样比push快
            // }
            return arr_1;
        }
    };
    /**
     * 转换金额格式
     * 如：10000 --> 10,000.00
     */
    Utils.toThousands = function (sum, dotNum) {
        if (dotNum === void 0) { dotNum = 2; }
        if (this.isEmptyStr(sum))
            return '';
        dotNum = dotNum > 0 && dotNum <= 20 ? dotNum : 2;
        sum = parseFloat((sum + '').replace(/[^\d\.-]/g, '')).toFixed(dotNum);
        var left = sum.split('.')[0], right = sum.split('.')[1], result = '';
        for (var i = 0; i < left.length; i++) {
            result += left[i] + ((i + 1) % 3 === 0 && (i + 1) !== left.length ? ',' : '');
        }
        return result + '.' + right;
    };
    /**
     * 生成32位的guid
     *
     * @return 32位guid字符串
     */
    Utils.getGUID = function (length) {
        if (length === void 0) { length = 32; }
        var guid = '';
        var num = '';
        for (var i = 1; i <= length; i++) {
            num = Math.floor(Math.random() * 16.0).toString(16); // toString(16)表示返回16进制
            guid += num;
            if ((i === 8) || (i === 12) || (i === 16) || (i === 20))
                guid += '-';
        }
        return guid;
    };
    /**
     * 根据输入秒数，获取倒计时
     *
     * @param {number} seconds
     * @return {*}
     */
    Utils.getCountdown = function (inputSeconds, showDay) {
        if (showDay === void 0) { showDay = false; }
        // 返回结果
        var result = {
            days: '',
            hours: '',
            minutes: '',
            seconds: ''
        };
        if (this.isEmptyStr(inputSeconds)) {
            return result;
        }
        var secNum = 0;
        var days = 0;
        var hours = 0;
        var minutes = 0;
        var seconds = 0;
        // 显示天
        if (showDay) {
            secNum = parseInt(inputSeconds.toString(), 10);
            days = Math.floor(secNum / 86400); // 秒 --> 天 (86400: 24 * 60 * 60)
            hours = Math.floor(secNum / 3600 - days * 24); // 3600: 60 * 60
            minutes = Math.floor(secNum / 60 - (hours * 60 + days * 1440)); // 1440: 24 * 60
            seconds = secNum - (minutes * 60 + hours * 3600 + days * 86400); // 3600: 60 * 60
        }
        else {
            secNum = parseInt(inputSeconds.toString(), 10);
            hours = Math.floor(secNum / 3600); // 3600: 60 * 60
            minutes = Math.floor(secNum / 60 - hours * 60);
            seconds = secNum - ((minutes * 60) + (hours * 3600));
        }
        // < 10为了补足两位数 如5 --> 05
        result.days = (days < 10) ? '0' + days : days.toString();
        result.hours = (hours < 10) ? '0' + hours : hours.toString();
        result.minutes = (minutes < 10) ? '0' + minutes : minutes.toString();
        result.seconds = (seconds < 10) ? '0' + seconds : seconds.toString();
        return result;
    };
    /**
     * 判断值是否是数字
     *
     * @param object
     * @return boolean
     */
    Utils.isNumber = function (object) {
        if (this.isEmptyObject(object))
            return false;
        if (object.match(/^[0-9]+$/) && isNaN(object) === false) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * 判断是否为整数
     *
     * @param object 需要判断的数字
     */
    Utils.isInteger = function (object) {
        if (this.isEmptyObject(object))
            return false;
        if (!this.isNumber(object)) {
            return false;
        }
        return Math.floor(object) === object;
    };
    /**
     * 输入内容是否是手机号码
     *
     * @param value 手机号码
     */
    Utils.isPhone = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        return !!value.match(/^((13[0-9])|(14[0-9])|(15[0-9])|(16[0-9])|(17[0-9])|(18[0-9])|(19[0-9]))+[0-9]{8}$/);
    };
    /**
     * 输入内容是否是邮箱
     *
     * @param value 邮箱
     */
    Utils.isEmail = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        if (value.match(/^(\w)+(\.\w+)*@(\w)+((\.\w+)+)$/))
            return true;
        return false;
    };
    /**
     * 输入内容是否是身份证格式
     *
     * @param value 身份证号码
     */
    Utils.isIDCard = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        var rule = /(^[1-9]\d{5}(18|19|([23]\d))\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]$)|(^[1-9]\d{5}\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\d{2}$)/;
        if (value.match(rule))
            return true;
        return false;
    };
    /**
     * 输入内容是否为中文
     *
     * @param value 字符串
     */
    Utils.isChinese = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        if (value.match(/^[(\u4e00-\u9fa5)]+$/))
            return true;
        return false;
    };
    /**
     * 输入内容是否为英文
     *
     * @param value 字符串
     */
    Utils.isEnglish = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        if (value.match(/^[0-9a-zA-Z\_\s]+$/))
            return true;
        return false;
    };
    /**
     * 输入内容是否包含特殊字符
     * 验证名称（不能含有特殊字符）, named
     *
     * @param value 字符串
     */
    Utils.isNamed = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        if (value.match(/^[(\u4e00-\u9fa5)0-9a-zA-Z\_\s]+$/))
            return true;
        return false;
    };
    /**
     * 验证多个以","隔开的数组名称
     * 验证名称（不能含有特殊字符）, named
     *
     * @param value 字符串
     */
    Utils.isMultiNamed = function (value) {
        if (this.isEmptyStr(value)) {
            return false;
        }
        if (value.match(/^(([(\u4e00-\u9fa5)0-9a-zA-Z\_\s]+)|([(\u4e00-\u9fa5)0-9a-zA-Z\_\s]+(\,|\，)?[(\u4e00-\u9fa5)0-9a-zA-Z\_\s]+))+$/))
            return true;
        return false;
    };
    /**
     * 将数字转成中文大写
     *
     * @param num
     * @return string
     */
    Utils.digitUppercase = function (num) {
        if (this.isEmptyStr(num))
            return '';
        // 确保为数字
        num = parseInt(num + '', 10);
        var fraction = ['角', '分'];
        var digit = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'];
        var unit = [
            ['元', '万', '亿'],
            ['', '拾', '佰', '仟']
        ];
        var head = num < 0 ? '负' : '';
        var str = '';
        num = Math.abs(num); // 取绝对值
        for (var i = 0; i < fraction.length; i++) {
            str += (digit[Math.floor(num * 10 * Math.pow(10, i)) % 10] + fraction[i]).replace(/零./, '');
        }
        str = str || '整';
        num = Math.floor(num);
        for (var i = 0; i < unit[0].length && num > 0; i++) {
            var p = '';
            for (var j = 0; j < unit[1].length && num > 0; j++) {
                p = digit[num % 10] + unit[1][j] + p;
                num = Math.floor(num / 10);
            }
            str = p.replace(/(零.)*零$/, '').replace(/^$/, '零') + unit[0][i] + str;
        }
        return head + str.replace(/(零.)*零元/, '元').replace(/(零.)+/g, '零').replace(/^整$/, '零元整');
    };
    /**
     * 数组分组
     * 目前用于表情包分组
     * 使用了Math.ceil() 四舍五入，取大值
     * example: 15.1 --> 16  15.6 --> 16  15 --> 15
     *
     * @param array 要分组的数组
     * @param groupNum 分组后的每个数组数量
     */
    Utils.arrayGroup = function (array, eachGroupNum) {
        if (eachGroupNum === void 0) { eachGroupNum = 24; }
        // 用于返回的数组
        var items = [];
        if (this.isEmptyObject(array)) {
            return items;
        }
        // 可以分为几组
        var groupNum = Math.ceil(array.length / eachGroupNum);
        for (var i = 0; i < groupNum; i++) {
            items.push(array.slice(i * eachGroupNum, (i + 1) * eachGroupNum));
        }
        return items;
    };
    /**
     * 获取随机字符串
     *
     * @static
     * @param {number} [length=32]
     * @param {Array<string>} [randomArray]
     * @return {string}
     */
    Utils.getRandomStr = function (length, randomArray) {
        if (length === void 0) { length = 32; }
        // 用于返回结果
        var randomStr = '';
        // 没有给定randomArray
        if (!randomArray || randomArray.length <= 0) {
            randomArray = 'ABCDEFGHIJKLMNPQRSTUVWXYZabcdefghijklmnpqrstuvwxyz123456789'.split(''); // 默认去掉了O和0
        }
        for (var i = 0; i < length; i++) {
            randomStr += randomArray[Math.floor(Math.random() * randomArray.length)];
        }
        return randomStr;
    };
    /**
     * 获取随机字符串
     * return Math.random().toString(36).substring(2);
     *
     * @Author JoyoDuan
     * @Date 2020/5/9
     * @Description:
     */
    Utils.getNonceStr = function (radix) {
        if (radix === void 0) { radix = 36; }
        return Math.random().toString(radix).substring(2);
    };
    /**
     * 内容是否包含敏感词汇
     *
     * @static
     * @param {string} str 检测的内容
     * @param {Array<string>} forbiddenArray 被禁止的词汇
     * @return {boolean} 是否包含敏感词汇 true: 包含该  false: 不包含
     */
    Utils.isSensitive = function (str, forbiddenArray) {
        if (this.isEmptyStr(str))
            return false;
        // 正则表达式
        var regExp = '';
        // 遍历敏感词汇数组
        forbiddenArray.forEach(function (value, index) {
            if (index === forbiddenArray.length - 1) {
                regExp += value;
            }
            else {
                regExp += value + '|';
            }
        });
        // 匹配是否包含敏感词汇
        var pattern = new RegExp(regExp, 'g');
        if (pattern.test(str)) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * 获取含有搜索条件的数组
     *
     * @static
     * @param {Array<any>} array 原数组
     * @param {string} delimiter 搜索条件的分隔符
     * @return {Array<any>} 转换后的数组
     */
    Utils.getSearchArray = function (array, delimiter, field) {
        if (delimiter === void 0) { delimiter = ','; }
        if (field === void 0) { field = 'searchCondition'; }
        if (this.isEmptyObject(array)) {
            return new Array();
        }
        // 支持搜索的条件
        var searchCondition = '';
        array.forEach(function (item) {
            searchCondition = ''; // 重置搜索条件
            for (var key in item) {
                searchCondition += item[key] + delimiter;
            }
            item[field] = searchCondition.substring(0, searchCondition.length - 1);
        });
        return array;
    };
    /**
     * 获取含有搜索条件的item
     *
     * @static
     * @param {any} item 原item
     * @param {string} delimiter 搜索条件的分隔符
     * @return {any} 转换后的item
     */
    Utils.getSearchItem = function (item, delimiter, field) {
        if (delimiter === void 0) { delimiter = ','; }
        if (field === void 0) { field = 'searchCondition'; }
        if (this.isEmptyObject(item)) {
            return {};
        }
        // 支持搜索的条件
        var searchCondition = '';
        for (var key in item) {
            searchCondition += item[key] + delimiter;
        }
        item[field] = searchCondition.substring(0, searchCondition.length - 1);
        return item;
    };
    /**
     * 根据日期获取星期几
     *
     * @static
     * @param {string} date 日期
     * @param weekDays 自定义星期数组
     * @return {string} 当前时间是星期几
     */
    Utils.getWeekDay = function (date, weekDays) {
        if (this.isEmptyObject(weekDays)) {
            weekDays = ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六'];
        }
        // 传入字符串
        return weekDays[this.getDate(date).getDay()];
    };
    /**
     * 获取身份证信息
     *
     * @param IDCard 身份证号
     * @param type 返回类型: all | birthday | age | sex， 默认为: all
     * @return {any} result: {birthday, age, sex} | birthday | age | sex
     */
    Utils.getIDCardInfo = function (IDCard, type) {
        if (type === void 0) { type = 'all'; }
        // 用于返回结果
        var result = {};
        if (this.isIDCard(IDCard)) {
            // 从身份证中获取到的相关信息
            var year = void 0, month = void 0, day = void 0, sex = void 0;
            year = IDCard.substring(6, 10);
            month = IDCard.substring(10, 12);
            day = IDCard.substring(12, 14);
            sex = IDCard.substr(16, 1);
            // 出生年月日
            var birthday = year + '-' + month + '-' + day;
            // 系统当前时间
            var currentDate = void 0, currentMonth = void 0, currentDay = void 0;
            currentDate = new Date();
            currentMonth = currentDate.getMonth() + 1; // 0 - 11
            currentDay = currentDate.getDate();
            // 年龄
            var age = currentDate.getFullYear() - parseInt(year, 10) - 1;
            // 比较月份，计算年龄
            if (parseInt(month, 10) < currentMonth || (parseInt(month, 10) === currentMonth && parseInt(day, 10) <= currentDay)) {
                age++;
            }
            // 性别
            if ((parseInt(sex, 10) % 2) === 0) {
                sex = 0; // 女
            }
            else {
                sex = 1; // 男
            }
            switch (type) {
                case 'birthday':
                    result = birthday;
                    break;
                case 'age':
                    result = age;
                    break;
                case 'sex':
                    result = sex;
                    break;
                default:
                    result = {
                        birthday: birthday,
                        age: age,
                        sex: sex
                    };
                    break;
            }
        }
        else {
            result = {};
        }
        return result;
    };
    /**
     * 扩展参数，返回userOptions
     * 注意，该方法会改变userOptions的源对象，如果不希望改变，请传入{...userOptions}或者isRelated: false
     * isRelated为false时，内部方法会调用Utils.deepClone方法，所以性能稍差一些
     *
     * @Author JoyoDuan
     * @Date 2020/5/14
     * @Description:
     * @param defaultOptions 默认配置
     * @param userOptions 用户配置
     * @param isRelated 默认为true，改变内容是否关联userOptions，如果为true，则传入的userOptions会改变
     * @returns isRelated为true返回userOptions对象，否则返回全新对象
     */
    Utils.extend = function (defaultOptions, userOptions, isRelated) {
        if (isRelated === void 0) { isRelated = true; }
        if (this.isEmptyObject(defaultOptions) && this.isEmptyObject(userOptions)) {
            return {};
        }
        if (this.isEmptyObject(userOptions))
            return defaultOptions;
        // 关联改变userOptions
        if (isRelated) {
            for (var key in defaultOptions) {
                if (this.isNull(userOptions[key])) {
                    userOptions[key] = defaultOptions[key];
                }
                else if (typeof userOptions[key] === 'object') {
                    // isRelated为true，有关联，会改变源对象，所以可以不赋值
                    this.extend(defaultOptions[key], userOptions[key], isRelated); // 深度匹配
                }
            }
            return userOptions;
        }
        else {
            // 拷贝新对象
            var userOpts = this.deepClone(userOptions);
            for (var key in defaultOptions) {
                if (this.isNull(userOpts[key])) {
                    userOpts[key] = defaultOptions[key];
                }
                else if (typeof userOpts[key] === 'object') {
                    // isRelated为false，没有关联，返回的是全新的对象，所以要赋值
                    userOpts[key] = this.extend(defaultOptions[key], userOpts[key], isRelated); // 深度匹配
                }
            }
            return userOpts;
        }
    };
    /**
     * 扩展基础（根基）参数，返回basicOptions
     * 注意，该方法会改变basicOptions的源对象，如果不希望改变，请传入{...basicOptions}或者isRelated: false
     * isRelated为false时，内部方法会调用Utils.deepClone方法，所以性能稍差一些
     *
     * @Author JoyoDuan
     * @Date 2020/5/14
     * @Description:
     * @param basicOptions 默认基础配置
     * @param userOptions 用户配置
     * @param isRelated 默认为true，改变内容是否关联userOptions，如果为true，则传入的userOptions会改变
     * @returns isRelated为true返回userOptions对象，否则返回全新对象
     */
    Utils.extendBasic = function (basicOptions, userOptions, isRelated) {
        if (isRelated === void 0) { isRelated = true; }
        if (this.isEmptyObject(basicOptions) && this.isEmptyObject(userOptions)) {
            return {};
        }
        if (this.isEmptyObject(userOptions))
            return basicOptions;
        // 关联改变userOptions
        if (isRelated) {
            for (var key in userOptions) {
                // 基本对象不存在的key不需要赋值
                if (this.isNull(basicOptions[key])) {
                    continue;
                }
                if (this.isNotNull(userOptions[key])) {
                    basicOptions[key] = userOptions[key];
                }
                else if (typeof userOptions[key] === 'object') {
                    // isRelated为true，有关联，会改变源对象，所以可以不赋值
                    this.extendBasic(basicOptions[key], userOptions[key], isRelated); // 深度匹配
                }
            }
            return basicOptions;
        }
        else {
            // 拷贝新对象
            var userOpts = this.deepClone(userOptions);
            for (var key in userOpts) {
                // 基本对象不存在的key不需要赋值
                if (this.isNull(basicOptions[key])) {
                    continue;
                }
                if (this.isNotNull(userOpts[key])) {
                    basicOptions[key] = userOpts[key];
                }
                else if (typeof userOpts[key] === 'object') {
                    // isRelated为false，没有关联，返回的是全新的对象，所以要赋值
                    basicOptions[key] = this.extendBasic(basicOptions[key], userOpts[key], isRelated); // 深度匹配
                }
            }
            return basicOptions;
        }
    };
    /**
     * 深度复制对象
     * 该方法不能拷贝 时间：Date，正则: RegExp，如有这些属性，请使用 deepClone()方法
     *
     * @param obj
     */
    Utils.deepCopy = function (obj) {
        if (this.isEmptyObject(obj)) {
            return {};
        }
        if (obj.constructor === Object && typeof obj === 'object') {
            // 存储返回的结果
            var result = Array.isArray(obj) ? [] : {};
            // 遍历
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (this.isNull(obj[key])) {
                        result[key] = obj[key];
                        continue;
                    }
                    // 对象
                    if (obj[key].constructor === Object) {
                        result[key] = this.deepCopy(obj[key]); // 递归深度复制
                    }
                    else {
                        result[key] = obj[key]; // 简单复制
                    }
                }
            }
            return result;
        }
        try {
            return JSON.parse(JSON.stringify(obj));
        }
        catch (e) {
            console.error('Error - JSON.parse(JSON.stringify(obj))', e);
            return obj; // 不是object，返回
        }
    };
    /**
     * 深度拷贝对象
     * 该方法支持拷贝 时间：Date，正则: RegExp
     *
     * @static
     * @param {*} obj
     * @return {*}
     */
    Utils.deepClone = function (obj) {
        if (this.isEmptyObject(obj)) {
            return {};
        }
        if (obj.constructor === Object && typeof obj === 'object') {
            // 存储返回的结果
            var result = Array.isArray(obj) ? [] : {};
            // 遍历
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (this.isNull(obj[key])) {
                        result[key] = obj[key];
                        continue;
                    }
                    switch (obj[key].constructor) {
                        case Array || Object:// 数组对象
                            result[key] = this.deepClone(obj[key]); // 深度拷贝
                            break;
                        case Date:// 时间格式
                            result[key] = this.getDate(obj[key]);
                            break;
                        case RegExp:// 正则表达
                            var flags = '';
                            flags += obj[key].global ? 'g' : '';
                            flags += obj[key].ignoreCase ? 'i' : '';
                            flags += obj[key].multiline ? 'm' : '';
                            result[key] = new RegExp(obj[key].source, flags);
                            break;
                        default:
                            result[key] = obj[key]; // 简单拷贝
                            break;
                    }
                }
            }
            return result;
        }
        try {
            return JSON.parse(JSON.stringify(obj));
        }
        catch (e) {
            console.error('Error - JSON.parse(JSON.stringify(obj))', e);
            return obj; // 不是object，返回
        }
    };
    /**
     * 在URL/URI后边添加后缀
     * 后缀格式：?Math.random() 如: https://www.baidu.com/JoyoDuan.jpg?0.88001
     *
     * @param url
     */
    Utils.tailURL = function (url) {
        if (this.isEmptyStr(url)) {
            return '';
        }
        return url.startsWith('http') ? (url + '?' + Math.random()) : url;
    };
    /**
     * 比较版本号
     * Utils.versionCompare('1.0.358_20180820090554', '1.0.358_20180820090553') = 1
     *
     * @static
     * @param {string} version1
     * @param {string} version2
     * @return {number} 0代表相等，1代表version1大，-1代表version2
     */
    Utils.versionCompare = function (version1, version2) {
        if (this.isEmptyStr(version1) || this.isEmptyStr(version2)) {
            return 0;
        }
        // 版本号相同
        if (version1 === version2) {
            return 0;
        }
        var version1Array = version1.split(/[._]/);
        var version2Array = version2.split(/[._]/);
        var index = 0; // 当前索引
        var minLength = Math.min(version1.length, version2.length); // 取最小的长度来遍历
        var diff = 0; // 比较的数字
        while (index < minLength && (diff = Number.parseInt(version1Array[index]) - Number.parseInt(version2Array[index])) === 0) {
            index++;
        }
        if (diff === 0) {
            for (var i = index; i < version1Array.length; i++) {
                if (Number.parseInt(version1Array[i]) > 0) {
                    return 1;
                }
            }
            for (var i = index; i < version2Array.length; i++) {
                if (Number.parseInt(version2Array[i]) > 0) {
                    return -1;
                }
            }
            return 0;
        }
        else {
            return diff > 0 ? 1 : -1;
        }
    };
    /**
     * 是否是手机环境
     *
     * @Author JoyoDuan
     * @Date 2020/4/14
     * @Description:
     */
    Utils.isMobile = function () {
        return this.isAndroid() || this.isIOS();
    };
    /**
     * 是否是ios设备
     *
     * @return true: 是  false: 否
     */
    Utils.isIOS = function () {
        return !!navigator.userAgent.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/); // 是否为ios设备
    };
    /**
     * 是否是Safari设备
     *
     * @return true: 是  false: 否
     */
    Utils.isSafari = function () {
        var u = navigator.userAgent;
        return /Safari/.test(u) && !/Chrome/.test(u);
    };
    /**
     * 是否是Android设备
     *
     * @return true: 是  false: 否
     */
    Utils.isAndroid = function () {
        var u = navigator.userAgent;
        return u.indexOf('Android') > -1 || u.indexOf('Adr') > -1; // 是否为android端
    };
    /**
     * 是否是PC环境
     *
     * @return true: 是  false: 否
     */
    Utils.isPC = function () {
        return typeof window.orientation === 'undefined'; // 是否为PC端
    };
    /**
     * 获取子元素
     *
     * @param element 元素
     * @param cssClass classList.contains(cssClass) 元素样式
     * @param selectors element.querySelector(selectors) 查找元素的样式
     *
     * @Author JoyoDuan
     * @Date 2019/12/16
     * @Description:
     */
    Utils.getChildElement = function (element, cssClass, selectors) {
        if (cssClass === void 0) { cssClass = 'ion-page'; }
        if (selectors === void 0) { selectors = ':scope > .ion-page, :scope > ion-nav, :scope > ion-tabs'; }
        if (this.isEmptyObject(element)) {
            return null;
        }
        // 传入的就是子元素
        if (Utils.isNotEmptyStr(cssClass) && element.classList.contains(cssClass)) {
            return element;
        }
        // 向下查找子元素
        var ChildElement = element.querySelector(selectors);
        if (ChildElement) {
            return ChildElement;
        }
        return element;
    };
    /**
     * 获取元素样式属性的值
     *
     * @param element 元素
     * @param styleName 属性 如：marginTop、height
     * @param pseudoElt getComputedStyle()中的第二个参数 指定一个要匹配的伪元素的字符串。必须对普通元素省略（或null）
     */
    Utils.getElementStyleValue = function (element, styleName, pseudoElt) {
        if (this.isEmptyObject(element) || this.isEmptyStr(styleName)) {
            return 0;
        }
        var attrValue = element.currentStyle ? element.currentStyle[styleName] : getComputedStyle(element, pseudoElt)[styleName];
        return Number.parseFloat(attrValue.replace(/[^0-9.]/, ''));
    };
    /**
     * 将字母转成小写
     *
     * @param str 需要转换的字符串
     */
    Utils.toLowerCase = function (str) {
        if (this.isEmptyStr(str)) {
            return '';
        }
        return str.toLowerCase();
    };
    /**
     * 将字母转成大写
     *
     * @param str 需要转换的字符串
     */
    Utils.toUpperCase = function (str) {
        if (this.isEmptyStr(str)) {
            return '';
        }
        return str.toUpperCase();
    };
    /**
     * 将字母转成小写
     * 少数语言(如土耳其语言)会为Unicode大小写转换应用特殊的规则
     *
     * @param str 需要转换的字符串
     */
    Utils.toLocaleLowerCase = function (str) {
        if (this.isEmptyStr(str)) {
            return '';
        }
        return str.toLocaleLowerCase();
    };
    /**
     * 将字母转成大写
     * 少数语言(如土耳其语言)会为Unicode大小写转换应用特殊的规则
     *
     * @param str 需要转换的字符串
     */
    Utils.toLocaleUpperCase = function (str) {
        if (this.isEmptyStr(str)) {
            return '';
        }
        return str.toLocaleUpperCase();
    };
    /**
     * 根据文件路径filePath获取文件名
     *
     * @param string path
     * @returns string
     */
    Utils.getFileName = function (path) {
        if (this.isEmptyStr(path)) {
            return '';
        }
        return path.substring(path.lastIndexOf('/') + 1);
    };
    /**
     * 根据文件路径获取文件后缀
     *
     * @param string path
     * @returns string
     */
    Utils.getFileSuffix = function (path) {
        if (this.isEmptyStr(path)) {
            return '';
        }
        return path.substring(path.lastIndexOf('.'));
    };
    /**
     * 计时器
     * 执行回调后会clearTimeout
     *
     * @param handler 执行函数
     * @param time 计时器时间
     * @Author JoyoDuan
     * @Date 2019/12/17
     * @Description:
     */
    Utils.setTimeout = function (handler, time) {
        if (time === void 0) { time = 0; }
        var timer = setTimeout(function () {
            handler();
            clearTimeout(timer);
        }, time);
    };
    /**
     * 自定义实现eval()方法
     * eval方法只能在非严格模式中进行使用，在use strict中是不允许使用这个方法的。
     *
     * @Author JoyoDuan
     * @Date 2020/6/11
     * @Description:
     */
    Utils.eval = function (x) {
        // 如果x不是字符串，则直接返回x
        if (this.isNull(x) || typeof x !== 'string')
            return x;
        return Function('return' + x)();
    };
    /**
     * 删除对象中的空属性（或键值对）
     *
     * @Author JoyoDuan
     * @Date 2020/7/7
     * @Description:
     */
    Utils.deleteEmptyProperty = function (obj) {
        var value;
        for (var key in obj) {
            value = obj[key];
            // null、undefined、''
            if (this.isEmptyStr(value)) {
                delete obj[key];
                continue;
            }
            // 对象
            if (typeof value === 'object') {
                // 数组
                if (Array.isArray(value) && value.length === 0) {
                    delete obj[key];
                    // 下一个循环
                    continue;
                }
                // 递归删除
                this.deleteEmptyProperty(value);
            }
        }
        return obj;
    };
    /**
     * 中文转Unicode
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    Utils.encodeUnicode = function (str) {
        var res = [];
        for (var i = 0; i < str.length; i++) {
            res[i] = ('00' + str.charCodeAt(i).toString(16)).slice(-4);
        }
        return '\\u' + res.join('\\u');
    };
    /**
     * Unicode转中文
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    Utils.decodeUnicode = function (str) {
        return unescape(str.replace(/\\u/gi, '%u'));
    };
    /**
     * Ascii 转 Unicode
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    Utils.A2U = function (str) {
        var reserved = '';
        for (var i = 0; i < str.length; i++) {
            reserved += '&#' + str.charCodeAt(i) + ';';
        }
        return reserved;
    };
    /**
     * Unicode 转 Ascii
     *
     * @Author JoyoDuan
     * @Date 2020/7/16
     * @Description:
     */
    Utils.U2A = function (str) {
        var reserved = '';
        var code = str.match(/&#(d+);/g);
        if (code === null) {
            return str;
        }
        for (var i = 0; i < code.length; i++) {
            reserved += String.fromCharCode(Number(code[i].replace(/[&#;]/g, '')));
        }
        return reserved;
    };
    /**
     * 每调用一次，sequence加1
     * @param sequence
     */
    Utils.getSequence = (function () {
        var sequence = 0;
        return function () {
            return ++sequence;
        };
    })();
    return Utils;
}());
export { Utils };
/** ----------------------------------------------------- Utils 工具类 end ----------------------------------------------------- */
//# sourceMappingURL=index.js.map